package node

import (
	"bytes"
	"fmt"
	"strings"
	"testing"

	"git.autistici.org/ale/autoradio"
	"github.com/aryann/difflib"
)

const (
	expectedSlaveConfig = `<!-- Automatically generated, do not edit -->

<icecast>
  <limits>
    <clients>1000</clients>
    <sources>500</sources>
    <queue-size>1048576</queue-size>
    <client-timeout>30</client-timeout>
    <header-timeout>15</header-timeout>
    <source-timeout>60</source-timeout>
    <burst-size>131072</burst-size>
  </limits>
  <authentication>
    <source-password>sourcepass</source-password>
    <admin-user>admin</admin-user>
    <admin-password>adminpass</admin-password>
  </authentication>
  <hostname>1.2.3.4</hostname>
  <fileserve>1</fileserve>
  <paths>
    <basedir>/usr/share/icecast2</basedir>
    <logdir>/var/log/icecast2</logdir>
    <webroot>/usr/share/icecast2/web</webroot>
    <adminroot>/usr/share/icecast2/admin</adminroot>
  </paths>
  <logging>
    <accesslog>access.log</accesslog>
    <errorlog>error.log</errorlog>
    <loglevel>3</loglevel>
    <logsize>10000</logsize>
  </logging>
  <security>
    <chroot>0</chroot>
  </security>
  <listen-socket>
    <bind-address>0.0.0.0</bind-address>
    <port>8000</port>
    <shoutcast-compat>0</shoutcast-compat>
  </listen-socket>
  <relay>
    <server>2.3.4.5</server>
    <port>8000</port>
    <mount>/_stream/stream1.ogg</mount>
    <local-mount>/_stream/stream1.ogg</local-mount>
    <username>user</username>
    <password>pass</password>
    <on-demand>1</on-demand>
    <relay-shoutcast-metadata>1</relay-shoutcast-metadata>
  </relay>
  <relay>
    <server>example.com</server>
    <port>80</port>
    <mount>/stream2.ogg</mount>
    <local-mount>/_stream/stream2.ogg</local-mount>
    <username></username>
    <password></password>
    <on-demand>1</on-demand>
    <relay-shoutcast-metadata>1</relay-shoutcast-metadata>
  </relay>
  <relay>
    <server>2.3.4.5</server>
    <port>8000</port>
    <mount>/_stream/stream3.mp3</mount>
    <local-mount>/_stream/stream3.mp3</local-mount>
    <username>user</username>
    <password>pass</password>
    <on-demand>1</on-demand>
    <relay-shoutcast-metadata>1</relay-shoutcast-metadata>
  </relay>
</icecast>`

	expectedMasterConfig = `<!-- Automatically generated, do not edit -->

<icecast>
  <limits>
    <clients>1000</clients>
    <sources>500</sources>
    <queue-size>1048576</queue-size>
    <client-timeout>30</client-timeout>
    <header-timeout>15</header-timeout>
    <source-timeout>60</source-timeout>
    <burst-size>131072</burst-size>
  </limits>
  <authentication>
    <source-password>sourcepass</source-password>
    <admin-user>admin</admin-user>
    <admin-password>adminpass</admin-password>
  </authentication>
  <hostname>1.2.3.4</hostname>
  <fileserve>1</fileserve>
  <paths>
    <basedir>/usr/share/icecast2</basedir>
    <logdir>/var/log/icecast2</logdir>
    <webroot>/usr/share/icecast2/web</webroot>
    <adminroot>/usr/share/icecast2/admin</adminroot>
  </paths>
  <logging>
    <accesslog>access.log</accesslog>
    <errorlog>error.log</errorlog>
    <loglevel>3</loglevel>
    <logsize>10000</logsize>
  </logging>
  <security>
    <chroot>0</chroot>
  </security>
  <listen-socket>
    <bind-address>0.0.0.0</bind-address>
    <port>8000</port>
    <shoutcast-compat>0</shoutcast-compat>
  </listen-socket>
  <relay>
    <server>example.com</server>
    <port>80</port>
    <mount>/stream2.ogg</mount>
    <local-mount>/_stream/stream2.ogg</local-mount>
    <username></username>
    <password></password>
    <on-demand>1</on-demand>
    <relay-shoutcast-metadata>1</relay-shoutcast-metadata>
  </relay>
  <mount>
    <mount-name>/_stream/stream1.ogg</mount-name>
    <username>user</username>
    <password>pass</password>
    <fallback-mount>/fallback.ogg</fallback-mount>
    <fallback-override>1</fallback-override>
    <hidden>0</hidden>
  </mount>
  <mount>
    <mount-name>/_stream/stream3.mp3</mount-name>
    <username>user</username>
    <password>pass</password>
    <hidden>0</hidden>
  </mount>
</icecast>`
)

// Create an icecastConfig object with known passwords.
func createTestIcecastConfig() *icecastConfig {
	ic := newIcecastConfig("1.2.3.4", 1000)
	ic.Auth.SourcePassword = "sourcepass"
	ic.Auth.AdminPassword = "adminpass"
	return ic
}

func checkStrings(t *testing.T, got, want string) {
	if got == want {
		return
	}
	var diffout bytes.Buffer
	for _, d := range difflib.Diff(strings.Split(want, "\n"), strings.Split(got, "\n")) {
		fmt.Fprintf(&diffout, "%s\n", d.String())
	}
	t.Errorf("unexpected result: (+ got, - want)\n\n%s\n", diffout.String())
}

func TestIcecastConfig(t *testing.T) {
	// Create a test config with a few different mount types.
	c := newClusterConfig()
	c.mounts["/stream1.ogg"] = &autoradio.Mount{
		Name:     "/stream1.ogg",
		Username: "user",
		Password: "pass",
		Fallback: "/fallback.ogg",
	}
	c.mounts["/stream2.ogg"] = &autoradio.Mount{
		Name:     "/stream2.ogg",
		RelayUrl: "http://example.com/stream2.ogg",
	}
	c.mounts["/stream3.mp3"] = &autoradio.Mount{
		Name:     "/stream3.mp3",
		Username: "user",
		Password: "pass",
		Transcoding: &autoradio.EncodingParams{
			SourceName: "/stream2.ogg",
			Format:     "mp3",
			BitRate:    64,
			SampleRate: 44100,
			Channels:   2,
		},
	}

	// Test a relay config.
	ice := createTestIcecastConfig()
	ice.Update(c, false, "2.3.4.5")
	output, err := ice.Encode()
	if err != nil {
		t.Fatal(err)
	}
	checkStrings(t, string(output), expectedSlaveConfig)

	// Test a master config.
	ice = createTestIcecastConfig()
	ice.Update(c, true, "2.3.4.5")
	output, err = ice.Encode()
	if err != nil {
		t.Fatal(err)
	}
	checkStrings(t, string(output), expectedMasterConfig)
}
